# -*- coding: utf-8 -*-
# @Time : 2022/5/30 23:22
# @Author : hanxinkong
# @File : hash.py
# @Description : hash摘要相关
# @Software : PyCharm
import hashlib


def md5(text, encoding: str = 'utf-8'):
    data = hashlib.md5(text.encode(encoding))
    return data.hexdigest()



import re
from typing import Optional, Any, List, Union


def regx_match(pattern, string, flags=0, first: bool = False, default: Optional[Any] = None):
    match_result = re.findall(pattern, string, flags=flags)
    if match_result and first is True:
        return match_result[0]
    elif match_result:
        return match_result
    else:
        return default


def for_to_regx_match(pattern_list: List, string, default: Optional[Any] = None, **kwargs):
    for pattern in pattern_list:
        value = regx_match(pattern, string, **kwargs)
        if value:
            return value

    return default


def regex_match(patterns: Union[List[str], str], string: str, flags=0, first: bool = False,
                default: Optional[Any] = None):
    """多条件正则匹配"""
    values = None
    if isinstance(patterns, str):
        patterns = [patterns]

    for pattern in patterns:
        values = re.findall(pattern, string, flags=flags)
        if values:
            break

    if values and first is True:
        return values[0]
    elif values:
        return values
    else:
        return default



# -*- coding: utf-8 -*-
# @Time : 2022/5/30 22:46
# @Author : hanxinkong
# @File : jsons.py
# @Description : json相关工具
# @Software : PyCharm
import json
import datetime

from typing import Union, Dict, List, Optional, Any

from jsonpath import jsonpath as super_jsonpath


class CJsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, datetime.date):
            return obj.strftime('%Y-%m-%d')
        else:
            return json.JSONEncoder.default(self, obj)


def format_json(src_json: Union[Dict, List], ensure_ascii: bool = False, indent: Optional[int] = 4, **kwargs) -> str:
    return json.dumps(src_json, ensure_ascii=ensure_ascii, indent=indent, cls=CJsonEncoder, **kwargs)


def jsonpath(src_data: Union[Dict, List], expr: Union[str, List[str]], default: Optional[Any] = None,
             first: bool = False):
    """
    jsonpath解析
    :param src_data: 解析对象
    :param expr: jsonpath,可选多个
    :param default: 未获取到返回默认值， 默认空字符串
    :param first: `True`只返回第一个， `False`返回全部
    :return: 解析值或者 default
    """
    values = None

    if isinstance(expr, str):
        expr = [expr]

    for i in expr:
        values = super_jsonpath(src_data, i)
        if values:
            break

    if values is False:
        return default

    if first is True:
        values = values[0]

    return values


def is_format_json(wait_json_string: str) -> bool:
    # noinspection PyBroadException
    try:
        json.loads(wait_json_string)
        return True
    except Exception as _:
        pass
    return False



import re
from typing import Union

__all__ = ['verify_ip_address', 'verify_domain_name', 'verify_port', 'verify_url']

ipaddr_regex = re.compile(r'^((25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(25[0-5]|2[0-4]\d|[01]?\d\d?)$')
domain_name_regex = re.compile(
    r'^(([a-zA-Z]{1})|([a-zA-Z]{1}[a-zA-Z]{1})|'
    r'([a-zA-Z]{1}[0-9]{1})|([0-9]{1}[a-zA-Z]{1})|'
    r'([a-zA-Z0-9][-_.a-zA-Z0-9]{0,61}[a-zA-Z0-9]))\.'
    r'([a-zA-Z]{2,13}|[a-zA-Z0-9-]{2,30}.[a-zA-Z]{2,3})$'
)
subdomain_name = re.compile(r'([a-zA-Z0-9\u4e00-\u9fa5]{1,61}\.){2,10}')
url_regex = re.compile(r'^https?:/{2}\w.+$')
port_regex = re.compile(r'\d+')


def verify_ip_address(address: str) -> bool:
    """验证ip地址合法性"""
    return True if ipaddr_regex.match(address) else False


def verify_domain_name(value: str) -> bool:
    """
    Return whether or not given value is a valid domain.
    If the value is valid domain name this function returns ``True``, otherwise False
    :param value: domain string to validate
    """
    if domain_name_regex.match(value):
        return True
    else:
        if re.findall(r'[\u4e00-\u9fa5]+', value):
            return True
        return False


def verify_subdomain_name(value) -> bool:
    """
    Return whether or not given value is a valid domain.
    If the value is valid domain name this function returns ``True``, otherwise False
    :param value: domain string to validate
    """
    if subdomain_name.match(value):
        return True
    return False


def verify_port(port: Union[int, str]) -> bool:
    """验证端口合法性"""

    if port_regex.match(str(port)):
        if all([
            int(port) > 1,
            int(port) < 65532
        ]):
            return True
    return False


def verify_url(url: str) -> bool:
    """验证url合法性"""
    verify_status = url_regex.match(url)
    return True if verify_status else False



# -*- coding: utf-8 -*-
# @Time : 2022/5/30 22:27
# @Author : hanxinkong
# @File : decorator.py
# @Description : 装饰器工具
# @Software : PyCharm
from typing import Any

import logging

logger = logging.getLogger(__name__)


def retry(num: int = 3, fail_msg: str = None, exceptions: Any = Exception):
    """
    异常重试 可针对指定异常进行重试
    :param num: 重试次数
    :param fail_msg: 重试次数
    :param exceptions: 指定异常
    :return:
    """

    def decorator(func):
        def make_decorator(*args, **kwargs):
            for i in range(num):
                try:
                    finally_func = func(*args, **kwargs)
                    return finally_func
                except exceptions as e:
                    logger.error(f'方法：{func.__name__} 错误：{e} line：{e.__traceback__.tb_lineno}')
                    logger.debug(f'方法：{func.__name__} 准备第{i + 1}次重试')

            if fail_msg is None:
                logger.error(f'方法：{func.__name__} 重试{num}次无效')
            else:
                logger.error(fail_msg)

        return make_decorator

    return decorator



from .date import (current_date, date_parse, before_day, after_day, between_day, timestamp)
from .hash import md5
from .jsons import jsonpath, format_json, is_format_json
from .re import regex_match, regx_match, for_to_regx_match
from .verify import verify_ip_address, verify_domain_name, verify_port, verify_url
from .data import cookie_to_dic, clear_value
from .decorator import retry

__version__ = "1.0.10"



from typing import Dict, Any, List, Union


def cookie_to_dic(cookie: str) -> Dict[str, Any]:
    if cookie:
        return {item.split('=')[0]: '='.join(item.split('=')[1:]) for item in cookie.split('; ')}
    return {}


def clear_value(data: Union[List, Dict], value=None):
    """递归清除所有嵌套字典和列表中的指定值"""
    if isinstance(data, dict):
        for key, val in list(data.items()):
            if val == value:
                del data[key]
            else:
                clear_value(val, value)
    elif isinstance(data, list):
        for item in data:
            if item == value:
                data.remove(item)
            else:
                clear_value(item, value)



# -*- coding: utf-8 -*-
# @Time : 2022/5/30 23:22
# @Author : hanxinkong
# @File : date.py
# @Description : 时间相关
# @Software : PyCharm
from typing import Union, Optional, List
from datetime import datetime, timedelta

import dateparser
import pytz

FmtDate = Union[datetime, str]
IntFmtDate = Union[datetime, int]
StrInt = Union[str, int]

all_timezones = ['Africa/Abidjan', 'Africa/Accra', 'Africa/Addis_Ababa', 'Africa/Algiers', 'Africa/Asmara',
                 'Africa/Asmera', 'Africa/Bamako', 'Africa/Bangui', 'Africa/Banjul', 'Africa/Bissau', 'Africa/Blantyre',
                 'Africa/Brazzaville', 'Africa/Bujumbura', 'Africa/Cairo', 'Africa/Casablanca', 'Africa/Ceuta',
                 'Africa/Conakry', 'Africa/Dakar', 'Africa/Dar_es_Salaam', 'Africa/Djibouti', 'Africa/Douala',
                 'Africa/El_Aaiun', 'Africa/Freetown', 'Africa/Gaborone', 'Africa/Harare', 'Africa/Johannesburg',
                 'Africa/Juba', 'Africa/Kampala', 'Africa/Khartoum', 'Africa/Kigali', 'Africa/Kinshasa', 'Africa/Lagos',
                 'Africa/Libreville', 'Africa/Lome', 'Africa/Luanda', 'Africa/Lubumbashi', 'Africa/Lusaka',
                 'Africa/Malabo', 'Africa/Maputo', 'Africa/Maseru', 'Africa/Mbabane', 'Africa/Mogadishu',
                 'Africa/Monrovia', 'Africa/Nairobi', 'Africa/Ndjamena', 'Africa/Niamey', 'Africa/Nouakchott',
                 'Africa/Ouagadougou', 'Africa/Porto-Novo', 'Africa/Sao_Tome', 'Africa/Timbuktu', 'Africa/Tripoli',
                 'Africa/Tunis', 'Africa/Windhoek', 'America/Adak', 'America/Anchorage', 'America/Anguilla',
                 'America/Antigua', 'America/Araguaina', 'America/Argentina/Buenos_Aires',
                 'America/Argentina/Catamarca', 'America/Argentina/ComodRivadavia', 'America/Argentina/Cordoba',
                 'America/Argentina/Jujuy', 'America/Argentina/La_Rioja', 'America/Argentina/Mendoza',
                 'America/Argentina/Rio_Gallegos', 'America/Argentina/Salta', 'America/Argentina/San_Juan',
                 'America/Argentina/San_Luis', 'America/Argentina/Tucuman', 'America/Argentina/Ushuaia',
                 'America/Aruba', 'America/Asuncion', 'America/Atikokan', 'America/Atka', 'America/Bahia',
                 'America/Bahia_Banderas', 'America/Barbados', 'America/Belem', 'America/Belize',
                 'America/Blanc-Sablon', 'America/Boa_Vista', 'America/Bogota', 'America/Boise', 'America/Buenos_Aires',
                 'America/Cambridge_Bay', 'America/Campo_Grande', 'America/Cancun', 'America/Caracas',
                 'America/Catamarca', 'America/Cayenne', 'America/Cayman', 'America/Chicago', 'America/Chihuahua',
                 'America/Ciudad_Juarez', 'America/Coral_Harbour', 'America/Cordoba', 'America/Costa_Rica',
                 'America/Creston', 'America/Cuiaba', 'America/Curacao', 'America/Danmarkshavn', 'America/Dawson',
                 'America/Dawson_Creek', 'America/Denver', 'America/Detroit', 'America/Dominica', 'America/Edmonton',
                 'America/Eirunepe', 'America/El_Salvador', 'America/Ensenada', 'America/Fort_Nelson',
                 'America/Fort_Wayne', 'America/Fortaleza', 'America/Glace_Bay', 'America/Godthab', 'America/Goose_Bay',
                 'America/Grand_Turk', 'America/Grenada', 'America/Guadeloupe', 'America/Guatemala',
                 'America/Guayaquil', 'America/Guyana', 'America/Halifax', 'America/Havana', 'America/Hermosillo',
                 'America/Indiana/Indianapolis', 'America/Indiana/Knox', 'America/Indiana/Marengo',
                 'America/Indiana/Petersburg', 'America/Indiana/Tell_City', 'America/Indiana/Vevay',
                 'America/Indiana/Vincennes', 'America/Indiana/Winamac', 'America/Indianapolis', 'America/Inuvik',
                 'America/Iqaluit', 'America/Jamaica', 'America/Jujuy', 'America/Juneau', 'America/Kentucky/Louisville',
                 'America/Kentucky/Monticello', 'America/Knox_IN', 'America/Kralendijk', 'America/La_Paz',
                 'America/Lima', 'America/Los_Angeles', 'America/Louisville', 'America/Lower_Princes', 'America/Maceio',
                 'America/Managua', 'America/Manaus', 'America/Marigot', 'America/Martinique', 'America/Matamoros',
                 'America/Mazatlan', 'America/Mendoza', 'America/Menominee', 'America/Merida', 'America/Metlakatla',
                 'America/Mexico_City', 'America/Miquelon', 'America/Moncton', 'America/Monterrey',
                 'America/Montevideo', 'America/Montreal', 'America/Montserrat', 'America/Nassau', 'America/New_York',
                 'America/Nipigon', 'America/Nome', 'America/Noronha', 'America/North_Dakota/Beulah',
                 'America/North_Dakota/Center', 'America/North_Dakota/New_Salem', 'America/Nuuk', 'America/Ojinaga',
                 'America/Panama', 'America/Pangnirtung', 'America/Paramaribo', 'America/Phoenix',
                 'America/Port-au-Prince', 'America/Port_of_Spain', 'America/Porto_Acre', 'America/Porto_Velho',
                 'America/Puerto_Rico', 'America/Punta_Arenas', 'America/Rainy_River', 'America/Rankin_Inlet',
                 'America/Recife', 'America/Regina', 'America/Resolute', 'America/Rio_Branco', 'America/Rosario',
                 'America/Santa_Isabel', 'America/Santarem', 'America/Santiago', 'America/Santo_Domingo',
                 'America/Sao_Paulo', 'America/Scoresbysund', 'America/Shiprock', 'America/Sitka',
                 'America/St_Barthelemy', 'America/St_Johns', 'America/St_Kitts', 'America/St_Lucia',
                 'America/St_Thomas', 'America/St_Vincent', 'America/Swift_Current', 'America/Tegucigalpa',
                 'America/Thule', 'America/Thunder_Bay', 'America/Tijuana', 'America/Toronto', 'America/Tortola',
                 'America/Vancouver', 'America/Virgin', 'America/Whitehorse', 'America/Winnipeg', 'America/Yakutat',
                 'America/Yellowknife', 'Antarctica/Casey', 'Antarctica/Davis', 'Antarctica/DumontDUrville',
                 'Antarctica/Macquarie', 'Antarctica/Mawson', 'Antarctica/McMurdo', 'Antarctica/Palmer',
                 'Antarctica/Rothera', 'Antarctica/South_Pole', 'Antarctica/Syowa', 'Antarctica/Troll',
                 'Antarctica/Vostok', 'Arctic/Longyearbyen', 'Asia/Aden', 'Asia/Almaty', 'Asia/Amman', 'Asia/Anadyr',
                 'Asia/Aqtau', 'Asia/Aqtobe', 'Asia/Ashgabat', 'Asia/Ashkhabad', 'Asia/Atyrau', 'Asia/Baghdad',
                 'Asia/Bahrain', 'Asia/Baku', 'Asia/Bangkok', 'Asia/Barnaul', 'Asia/Beirut', 'Asia/Bishkek',
                 'Asia/Brunei', 'Asia/Calcutta', 'Asia/Chita', 'Asia/Choibalsan', 'Asia/Chongqing', 'Asia/Chungking',
                 'Asia/Colombo', 'Asia/Dacca', 'Asia/Damascus', 'Asia/Dhaka', 'Asia/Dili', 'Asia/Dubai',
                 'Asia/Dushanbe', 'Asia/Famagusta', 'Asia/Gaza', 'Asia/Harbin', 'Asia/Hebron', 'Asia/Ho_Chi_Minh',
                 'Asia/Hong_Kong', 'Asia/Hovd', 'Asia/Irkutsk', 'Asia/Istanbul', 'Asia/Jakarta', 'Asia/Jayapura',
                 'Asia/Jerusalem', 'Asia/Kabul', 'Asia/Kamchatka', 'Asia/Karachi', 'Asia/Kashgar', 'Asia/Kathmandu',
                 'Asia/Katmandu', 'Asia/Khandyga', 'Asia/Kolkata', 'Asia/Krasnoyarsk', 'Asia/Kuala_Lumpur',
                 'Asia/Kuching', 'Asia/Kuwait', 'Asia/Macao', 'Asia/Macau', 'Asia/Magadan', 'Asia/Makassar',
                 'Asia/Manila', 'Asia/Muscat', 'Asia/Nicosia', 'Asia/Novokuznetsk', 'Asia/Novosibirsk', 'Asia/Omsk',
                 'Asia/Oral', 'Asia/Phnom_Penh', 'Asia/Pontianak', 'Asia/Pyongyang', 'Asia/Qatar', 'Asia/Qostanay',
                 'Asia/Qyzylorda', 'Asia/Rangoon', 'Asia/Riyadh', 'Asia/Saigon', 'Asia/Sakhalin', 'Asia/Samarkand',
                 'Asia/Seoul', 'Asia/Shanghai', 'Asia/Singapore', 'Asia/Srednekolymsk', 'Asia/Taipei', 'Asia/Tashkent',
                 'Asia/Tbilisi', 'Asia/Tehran', 'Asia/Tel_Aviv', 'Asia/Thimbu', 'Asia/Thimphu', 'Asia/Tokyo',
                 'Asia/Tomsk', 'Asia/Ujung_Pandang', 'Asia/Ulaanbaatar', 'Asia/Ulan_Bator', 'Asia/Urumqi',
                 'Asia/Ust-Nera', 'Asia/Vientiane', 'Asia/Vladivostok', 'Asia/Yakutsk', 'Asia/Yangon',
                 'Asia/Yekaterinburg', 'Asia/Yerevan', 'Atlantic/Azores', 'Atlantic/Bermuda', 'Atlantic/Canary',
                 'Atlantic/Cape_Verde', 'Atlantic/Faeroe', 'Atlantic/Faroe', 'Atlantic/Jan_Mayen', 'Atlantic/Madeira',
                 'Atlantic/Reykjavik', 'Atlantic/South_Georgia', 'Atlantic/St_Helena', 'Atlantic/Stanley',
                 'Australia/ACT', 'Australia/Adelaide', 'Australia/Brisbane', 'Australia/Broken_Hill',
                 'Australia/Canberra', 'Australia/Currie', 'Australia/Darwin', 'Australia/Eucla', 'Australia/Hobart',
                 'Australia/LHI', 'Australia/Lindeman', 'Australia/Lord_Howe', 'Australia/Melbourne', 'Australia/NSW',
                 'Australia/North', 'Australia/Perth', 'Australia/Queensland', 'Australia/South', 'Australia/Sydney',
                 'Australia/Tasmania', 'Australia/Victoria', 'Australia/West', 'Australia/Yancowinna', 'Brazil/Acre',
                 'Brazil/DeNoronha', 'Brazil/East', 'Brazil/West', 'CET', 'CST6CDT', 'Canada/Atlantic',
                 'Canada/Central', 'Canada/Eastern', 'Canada/Mountain', 'Canada/Newfoundland', 'Canada/Pacific',
                 'Canada/Saskatchewan', 'Canada/Yukon', 'Chile/Continental', 'Chile/EasterIsland', 'Cuba', 'EET', 'EST',
                 'EST5EDT', 'Egypt', 'Eire', 'Etc/GMT', 'Etc/GMT+0', 'Etc/GMT+1', 'Etc/GMT+10', 'Etc/GMT+11',
                 'Etc/GMT+12', 'Etc/GMT+2', 'Etc/GMT+3', 'Etc/GMT+4', 'Etc/GMT+5', 'Etc/GMT+6', 'Etc/GMT+7',
                 'Etc/GMT+8', 'Etc/GMT+9', 'Etc/GMT-0', 'Etc/GMT-1', 'Etc/GMT-10', 'Etc/GMT-11', 'Etc/GMT-12',
                 'Etc/GMT-13', 'Etc/GMT-14', 'Etc/GMT-2', 'Etc/GMT-3', 'Etc/GMT-4', 'Etc/GMT-5', 'Etc/GMT-6',
                 'Etc/GMT-7', 'Etc/GMT-8', 'Etc/GMT-9', 'Etc/GMT0', 'Etc/Greenwich', 'Etc/UCT', 'Etc/UTC',
                 'Etc/Universal', 'Etc/Zulu', 'Europe/Amsterdam', 'Europe/Andorra', 'Europe/Astrakhan', 'Europe/Athens',
                 'Europe/Belfast', 'Europe/Belgrade', 'Europe/Berlin', 'Europe/Bratislava', 'Europe/Brussels',
                 'Europe/Bucharest', 'Europe/Budapest', 'Europe/Busingen', 'Europe/Chisinau', 'Europe/Copenhagen',
                 'Europe/Dublin', 'Europe/Gibraltar', 'Europe/Guernsey', 'Europe/Helsinki', 'Europe/Isle_of_Man',
                 'Europe/Istanbul', 'Europe/Jersey', 'Europe/Kaliningrad', 'Europe/Kiev', 'Europe/Kirov', 'Europe/Kyiv',
                 'Europe/Lisbon', 'Europe/Ljubljana', 'Europe/London', 'Europe/Luxembourg', 'Europe/Madrid',
                 'Europe/Malta', 'Europe/Mariehamn', 'Europe/Minsk', 'Europe/Monaco', 'Europe/Moscow', 'Europe/Nicosia',
                 'Europe/Oslo', 'Europe/Paris', 'Europe/Podgorica', 'Europe/Prague', 'Europe/Riga', 'Europe/Rome',
                 'Europe/Samara', 'Europe/San_Marino', 'Europe/Sarajevo', 'Europe/Saratov', 'Europe/Simferopol',
                 'Europe/Skopje', 'Europe/Sofia', 'Europe/Stockholm', 'Europe/Tallinn', 'Europe/Tirane',
                 'Europe/Tiraspol', 'Europe/Ulyanovsk', 'Europe/Uzhgorod', 'Europe/Vaduz', 'Europe/Vatican',
                 'Europe/Vienna', 'Europe/Vilnius', 'Europe/Volgograd', 'Europe/Warsaw', 'Europe/Zagreb',
                 'Europe/Zaporozhye', 'Europe/Zurich', 'GB', 'GB-Eire', 'GMT', 'GMT+0', 'GMT-0', 'GMT0', 'Greenwich',
                 'HST', 'Hongkong', 'Iceland', 'Indian/Antananarivo', 'Indian/Chagos', 'Indian/Christmas',
                 'Indian/Cocos', 'Indian/Comoro', 'Indian/Kerguelen', 'Indian/Mahe', 'Indian/Maldives',
                 'Indian/Mauritius', 'Indian/Mayotte', 'Indian/Reunion', 'Iran', 'Israel', 'Jamaica', 'Japan',
                 'Kwajalein', 'Libya', 'MET', 'MST', 'MST7MDT', 'Mexico/BajaNorte', 'Mexico/BajaSur', 'Mexico/General',
                 'NZ', 'NZ-CHAT', 'Navajo', 'PRC', 'PST8PDT', 'Pacific/Apia', 'Pacific/Auckland',
                 'Pacific/Bougainville', 'Pacific/Chatham', 'Pacific/Chuuk', 'Pacific/Easter', 'Pacific/Efate',
                 'Pacific/Enderbury', 'Pacific/Fakaofo', 'Pacific/Fiji', 'Pacific/Funafuti', 'Pacific/Galapagos',
                 'Pacific/Gambier', 'Pacific/Guadalcanal', 'Pacific/Guam', 'Pacific/Honolulu', 'Pacific/Johnston',
                 'Pacific/Kanton', 'Pacific/Kiritimati', 'Pacific/Kosrae', 'Pacific/Kwajalein', 'Pacific/Majuro',
                 'Pacific/Marquesas', 'Pacific/Midway', 'Pacific/Nauru', 'Pacific/Niue', 'Pacific/Norfolk',
                 'Pacific/Noumea', 'Pacific/Pago_Pago', 'Pacific/Palau', 'Pacific/Pitcairn', 'Pacific/Pohnpei',
                 'Pacific/Ponape', 'Pacific/Port_Moresby', 'Pacific/Rarotonga', 'Pacific/Saipan', 'Pacific/Samoa',
                 'Pacific/Tahiti', 'Pacific/Tarawa', 'Pacific/Tongatapu', 'Pacific/Truk', 'Pacific/Wake',
                 'Pacific/Wallis', 'Pacific/Yap', 'Poland', 'Portugal', 'ROC', 'ROK', 'Singapore', 'Turkey', 'UCT',
                 'US/Alaska', 'US/Aleutian', 'US/Arizona', 'US/Central', 'US/East-Indiana', 'US/Eastern', 'US/Hawaii',
                 'US/Indiana-Starke', 'US/Michigan', 'US/Mountain', 'US/Pacific', 'US/Samoa', 'UTC', 'Universal',
                 'W-SU', 'WET', 'Zulu']

default_fmt_scheme = '%Y-%m-%d %H:%M:%S'


def before_day(src: Union[datetime, str], is_fmt: bool = True, scheme: str = None) -> FmtDate:
    """昨天"""
    assert src, 'Time format error'

    cur_time = date_parse(src, is_fmt=False)
    bf_day = cur_time - timedelta(days=1)
    if is_fmt:
        bf_day = format_date(bf_day, scheme)

    return bf_day


def after_day(src: Union[datetime, str], is_fmt: bool = True, scheme: str = None) -> FmtDate:
    """明天"""
    assert src, 'Time format error'

    cur_time = date_parse(src, is_fmt=False)
    af_day = cur_time + timedelta(days=1)
    if is_fmt:
        af_day = format_date(af_day, scheme)

    return af_day


def between_day(start: FmtDate, end: FmtDate, is_fmt: bool = True, scheme: str = '%Y-%m-%d') -> List[FmtDate]:
    """生成两个时间之间的时间"""
    assert start, 'Start time format error'

    days = []

    begin_date = date_parse(start, is_fmt=False)
    end_date = date_parse(end, is_fmt=False)

    while begin_date <= end_date:
        date = begin_date
        if is_fmt:
            date = format_date(begin_date, scheme)
        days.append(date)
        begin_date += timedelta(days=1)

    return days


def date_parse(obj: FmtDate, is_fmt: bool = True, scheme: str = None, select: Optional[str] = None,
               default_time_scheme: Optional[str] = '00:00:00', timezone: str = None) -> FmtDate:
    """文字时间解析成datetime对象，可选再次格式化，指定保留日期/时间（可设置默认值）部分，可指定时区"""

    assert obj, 'Time format error'

    obj_date = dateparser.parse(obj)

    if timezone:
        obj_date = timezone_(obj, is_fmt=False, timezone=timezone)

    if 'date' == select:
        scheme = f'%Y-%m-%d'
        obj_date = obj_date.date()
    if 'time' == select:
        scheme = default_time_scheme if default_time_scheme else f'%H:%M:%S'
        obj_date = obj_date.time()

    if default_time_scheme:
        scheme = f'%Y-%m-%d {default_time_scheme}'

    if is_fmt:
        obj_date = format_date(obj=obj_date, scheme=scheme)

    return obj_date


def timezone_(obj: FmtDate, is_fmt: bool = True, timezone: str = 'Asia/Shanghai', scheme: str = None) -> FmtDate:
    """时区转换"""
    assert obj, 'Time format error'
    obj_date = dateparser.parse(obj)

    tz = pytz.timezone(timezone)
    obj_date = obj_date.astimezone(tz)
    if is_fmt:
        obj_date = format_date(obj=obj_date, scheme=scheme)

    return obj_date


def format_date(obj: datetime.now, scheme: str = None) -> str:
    if scheme is None:
        scheme = default_fmt_scheme
    return obj.strftime(scheme)


def current_date(is_fmt: bool = True, scheme: str = None) -> FmtDate:
    """当前时间对象，默认为格式化标准文本时间格式"""
    cur_time = datetime.now()
    if is_fmt:
        cur_time = format_date(obj=cur_time, scheme=scheme)

    return cur_time


# def date_parse(obj: str, src_scheme: str = default_fmt_scheme, is_fmt: bool = True, scheme: str = None) -> FmtDate:
#     """文字时间解析成datetime对象，可选再次格式化"""
#     obj_date = datetime.strptime(obj, src_scheme)
#     if is_fmt:
#         obj_date = format_date(obj=obj_date, scheme=scheme)
#     return obj_date


def timestamp(ms: bool = False) -> int:
    """当前时间戳生成，默认秒，可选毫秒"""
    cur_time = current_date(is_fmt=False).timestamp()
    if ms:
        cur_time = cur_time * 1000

    return round(cur_time)


def timestamp_parse(obj: StrInt, is_fmt: bool = True, scheme: str = None) -> IntFmtDate:
    """时间戳解析为datetime对象，默认为格式化标准文本时间格式"""
    if isinstance(obj, str):
        obj = int(obj)

    obj_date = datetime.fromtimestamp(obj)
    if is_fmt:
        obj_date = format_date(obj=obj_date, scheme=scheme)

    return obj_date
